import sys
import wx
import wx.xrc as xrc
import listctrl  as  listmix
from   gui import images
import gui.tree
import gui.iPodderWindows
from   ipodder.contrib import feedparser
import platform
import os.path
from   localization import LanguageModule
import sys
if sys.version_info[0] == 2 and sys.version_info[1] <= 3:
    import webbrowser
else:
    from ipodder.contrib import webbrowser

# Skinning
from   gui.skin import STRIPE_EVEN_COLOR, STRIPE_ODD_COLOR

class PrefsDialog(wx.Dialog):
    def __init__(self):
        p = wx.PreDialog()
        self.PostCreate(p)

    def _(self,key):
        return self.m_stringtable.GetText(self.m_currentlanguage,key)
        
    def ValueControl(self, id, label):
        xrc.XRCCTRL(self, id).SetLabel(self._(label))
        
        
    def SetLanguages(self):
        GENERAL = 0
        THREADS = 1
        NETWORKING = 2
        PLAYER = 3
        FEEDMANAGER = 4
        ADVANCED = 5
        
        self.SetLabel(self._("str_preferences"))
        
        tabsdict = {
            FEEDMANAGER : "str_feed_manager",
            GENERAL : "str_general",
            THREADS : "str_threads",
            NETWORKING : "str_networking",
            PLAYER : "str_player",
            ADVANCED : "str_advanced"
        }
        en_loaded = False
        for key in tabsdict.keys():
            try:
                self.notebook.SetPageText(key, self._(tabsdict[key]))
            except AssertionError:
                """Unfortunately wide characters and Notebooks don't mix well
                on Mac, so we fall back to English."""
                if not en_loaded:
                    self.m_stringtable.LoadLanguage(LanguageModule.ENGLISH)
                    en_loaded = True
                self.notebook.SetPageText(key,self.m_stringtable.GetText(LanguageModule.ENGLISH,tabsdict[key]))

        self.ValueControl("PRF_FEEDMANAGER_BTN_PODNOVA", "str_feedmanager_btn_podnova")
        self.ValueControl("PRF_FEEDMANAGER_BTN_GIGADIAL", "str_feedmanager_btn_gigadial")
        self.ValueControl("PRF_TXT_FEEDMANAGER", "str_txt_feedmanager")
        self.ValueControl("PRF_FEEDMANAGER_ENABLE", "str_feedmanager_enable")
        self.ValueControl("PRF_LBL_GEN_OPTIONS", "str_gen_options_expl")
        self.ValueControl("HIDE_ON_STARTUP", "str_hide_on_startup")
        self.ValueControl("HIDE_TRAY_ICON", "str_hide_tray_icon")
        self.ValueControl("SCAN_ON_STARTUP", "str_run_check_startup")
        self.ValueControl("CHKUPDATE_ON_STARTUP", "str_chkupdate_on_startup")
        self.ValueControl("PLAY_ON_DOWNLOAD", "str_play_after_download")
        self.ValueControl("PRF_TEXT_PLAYERNAME", "str_choose_a_player")
        self.ValueControl("PRF_LBL_STORAGE_LOC_STORAGE", "str_location_and_storage")
        self.ValueControl("PRF_LBL_STORAGE_STOP", "str_stop_downloading")
        self.ValueControl("PRF_LBL_DOWNLOAD_FOLDER", "str_download_folder")
        self.ValueControl("PREFSBROWSE", "str_browse")
        
        self.ValueControl("PRF_MULTIPLE_DOWNLOAD", "str_multiple_download")
        self.ValueControl("PRF_MAX_FEEDSCANS", "str_max_feedscans")
        self.ValueControl("PRF_MAX_DOWNLOADS", "str_max_downloads")

        self.ValueControl("PRFCORALIZING", "str_coralize_urls")
        self.ValueControl("PRFUSEPROXY", "str_proxy_server")
        self.ValueControl("PRF_PROXY_ADDRESS", "str_proxy_address")
        self.ValueControl("PRF_PROXY_PORT", "str_proxy_port")
        self.ValueControl("PRF_PROXY_USERNAME", "str_username")	
        self.ValueControl("PRF_PROXY_PASSWORD", "str_password")

        self.ValueControl("PRF_POWER_USER", "str_options_power_users")
        self.ValueControl("DLCOMMANDENABLE", "str_run_command_download")
        self.ValueControl("PRF_RCMD_FULL_PATH", "str_rcmd_full_path")
        self.ValueControl("PRF_RCMD_PODCAST_NAME", "str_rcmd_podcast_name")

        self.ValueControl("PRF_OTHER_ADV_OPTION", "str_other_advanced_options")
        self.ValueControl("SHOWLOGPAGE", "str_show_log")

        self.ValueControl("PREFSSAVE", "str_save")
        self.ValueControl("PREFSCANCEL", "str_cancel")
        

        #"No player"
	# Not needed anymore -- DO NOT USE
        #self.playerbox.SetItemLabel(1,self.m_stringtable.GetText(self.m_currentlanguage,"str_no_player"))
        
    def OnEnableProxy(self, event):
        if self.enableProxy.IsChecked()==True:
            self.proxyPort.Enable(True)
            self.proxyHost.Enable(True)
            self.proxyUsername.Enable(True)	
            self.proxyPassword.Enable(True)
        else:
            self.proxyPort.Enable(False)
            self.proxyHost.Enable(False)
            self.proxyUsername.Enable(False)	
            self.proxyPassword.Enable(False)	
        self.OnPrefChange(event)
    
    def HideOnStartup(self, event):
	#shade the hide tray icon or not
        if self.hideOnStartup.IsChecked()==True:
            self.hideTrayIcon.Enable(False)
        else:
            self.hideTrayIcon.Enable(True)
        self.OnPrefChange(event)
    
    def OnBtnPodNova(self, event):
        webbrowser.open("http://my.podnova.com")
    
    def OnBtnGigaDial(self, event):
        webbrowser.open("http://www.gigadial.net/public")
    
    def Init(self,ipodder,gui):
        self.ipodder = ipodder
        self.gui = gui
        self.m_currentlanguage = self.ipodder.config.screen_language
        self.m_stringtable = LanguageModule.StringTable(self.m_currentlanguage)

        self.player_type = xrc.XRCCTRL(self, "PRF_PLAYER")
        self.notebook = xrc.XRCCTRL(self, "NOTEBOOK")
        self.hideOnStartup = xrc.XRCCTRL(self, "HIDE_ON_STARTUP")
        self.hideTrayIcon = xrc.XRCCTRL(self, "HIDE_TRAY_ICON")
        self.scanOnStartup = xrc.XRCCTRL(self, "SCAN_ON_STARTUP")
        self.chkupdateOnStartup = xrc.XRCCTRL(self, "CHKUPDATE_ON_STARTUP")
        self.playOnDownload = xrc.XRCCTRL(self, "PLAY_ON_DOWNLOAD")
        self.dlCommandEnable = xrc.XRCCTRL(self, "DLCOMMANDENABLE")
        self.dlCommand = xrc.XRCCTRL(self, "DLCOMMAND")
        self.playerCommand = xrc.XRCCTRL(self, "PLAYERCOMMAND")
        self.showLogPage = xrc.XRCCTRL(self, "SHOWLOGPAGE")
        self.prefsSave = xrc.XRCCTRL(self, "PREFSSAVE")
        self.prefsCancel = xrc.XRCCTRL(self, "PREFSCANCEL")
        self.downloaddir = xrc.XRCCTRL(self, "PREFSDOWNLOADDIR")
        self.browse = xrc.XRCCTRL(self,"PREFSBROWSE")
        self.numFeedScanThreads = xrc.XRCCTRL(self, "NUMFEEDSCANTHREADS")
        self.numDownloadThreads = xrc.XRCCTRL(self, "NUMDOWNLOADTHREADS")
        self.maxHarddiscSizeMB = xrc.XRCCTRL(self, "MAX_HARDDISC_SIZE_MB")
        self.playerbox = xrc.XRCCTRL(self, "PRF_PLAYER")
        self.feedmanager_btn_podnova = xrc.XRCCTRL(self, "PRF_FEEDMANAGER_BTN_PODNOVA")
        self.feedmanager_btn_gigadial = xrc.XRCCTRL(self, "PRF_FEEDMANAGER_BTN_GIGADIAL")
        self.feedmanager_enable = xrc.XRCCTRL(self, "PRF_FEEDMANAGER_ENABLE")
        self.feedmanagerOPML = xrc.XRCCTRL(self, "PRF_FEEDMANAGER_OPML")
        self.feedmanagerLbl = xrc.XRCCTRL(self, "LBL_FEEDMANAGER_OPML_URL")
        self.feedmanagerOPML.SetDropTarget(MyURLDropTarget(self.feedmanagerOPML))

        # Networking
        self.corlizeUrls = xrc.XRCCTRL(self, "PRFCORALIZING")
        self.corlizeUrls.SetValue(self.ipodder.config.coralize_urls)

        self.enableProxy = xrc.XRCCTRL(self, "PRFUSEPROXY")
        self.proxyHost = xrc.XRCCTRL(self, "TXTPRFPROXYHOST")
        self.proxyPort = xrc.XRCCTRL(self, "TXTPRFPROXYPORT")
        self.proxyUsername = xrc.XRCCTRL(self, "TXTPRFPROXYUSERNAME")	
        self.proxyPassword = xrc.XRCCTRL(self, "TXTPRFPROXYPASSWORD")

        self.Bind(wx.EVT_CHECKBOX, self.HideOnStartup, self.hideOnStartup)
        self.Bind(wx.EVT_CHECKBOX, self.OnPrefChange, self.hideTrayIcon)
        self.Bind(wx.EVT_CHECKBOX, self.OnPrefChange, self.scanOnStartup)
        self.Bind(wx.EVT_CHECKBOX, self.OnPrefChange, self.chkupdateOnStartup)
        self.Bind(wx.EVT_CHECKBOX, self.OnPrefChange, self.playOnDownload)
        self.Bind(wx.EVT_CHECKBOX, self.OnPrefChange, self.dlCommandEnable)
        self.Bind(wx.EVT_COMBOBOX, self.OnPrefChange, self.numFeedScanThreads)
        self.Bind(wx.EVT_COMBOBOX, self.OnPrefChange, self.numDownloadThreads)
        self.Bind(wx.EVT_TEXT, self.OnPrefChange, self.maxHarddiscSizeMB)
        self.Bind(wx.EVT_TEXT, self.OnPrefChange, self.dlCommand)
        self.Bind(wx.EVT_TEXT, self.OnPrefChange, self.playerCommand)
        self.Bind(wx.EVT_CHECKBOX, self.OnPrefChange, self.showLogPage)
        self.Bind(wx.EVT_BUTTON, self.OnPrefsSave, self.prefsSave)
        self.Bind(wx.EVT_BUTTON, self.OnPrefsCancel, self.prefsCancel)
        self.Bind(wx.EVT_TEXT, self.OnPrefChange, self.downloaddir)
        self.Bind(wx.EVT_BUTTON, self.OnPrefsBrowse, self.browse)
        self.Bind(wx.EVT_CHECKBOX, self.OnPrefChange, self.corlizeUrls)
        self.Bind(wx.EVT_RADIOBOX, self.OnPrefChange, self.playerbox)
        self.Bind(wx.EVT_CHECKBOX, self.OnEnableProxy, self.enableProxy)
        self.Bind(wx.EVT_TEXT, self.OnPrefChange, self.proxyHost)
        self.Bind(wx.EVT_TEXT, self.OnPrefChange, self.proxyPort)
        self.Bind(wx.EVT_TEXT, self.OnPrefChange, self.proxyUsername)	
        self.Bind(wx.EVT_TEXT, self.OnPrefChange, self.proxyPassword)
        self.Bind(wx.EVT_TEXT, self.OnPrefChange, self.feedmanagerOPML)
        self.Bind(wx.EVT_CHECKBOX, self.OnFeedmanagerEnableChk, self.feedmanager_enable)
        self.Bind(wx.EVT_BUTTON, self.OnBtnPodNova, self.feedmanager_btn_podnova)
        self.Bind(wx.EVT_BUTTON, self.OnBtnGigaDial, self.feedmanager_btn_gigadial)
        self.feedmanagerOPML.SetDropTarget(MyURLDropTarget(self.feedmanagerOPML))

        self.ResetPrefs()        
        self.SetLanguages()

        self.SetAcceleratorTable(
            wx.AcceleratorTable([(wx.ACCEL_NORMAL, wx.WXK_ESCAPE, xrc.XRCID("PREFSCANCEL"))
            ]))
            
    def ResetPrefs(self):
        self.hideOnStartup.SetValue(self.ipodder.config.hide_on_startup)
        if self.hideOnStartup is True:
	    self.hideTrayIcon.Enable(False)
        self.scanOnStartup.SetValue(self.ipodder.config.scan_on_startup)
        self.chkupdateOnStartup.SetValue(self.ipodder.config.chkupdate_on_startup)
        self.playOnDownload.SetValue(self.ipodder.config.play_on_download)
        self.dlCommandEnable.SetValue(self.ipodder.config.dl_command_enable)
        self.dlCommand.SetValue(self.ipodder.config.dl_command)
        self.playerCommand.SetValue(self.ipodder.config.player_command)
        self.showLogPage.SetValue(self.ipodder.config.show_log_page)
        self.downloaddir.SetValue(unicode(self.ipodder.config.download_dir))
        self.corlizeUrls.SetValue(self.ipodder.config.coralize_urls)
        self.feedmanager_enable.SetValue(self.ipodder.config.feedmanager_enable)
        self.feedmanagerOPML.SetValue(self.ipodder.config.feedmanager_opml_url)
        self.feedmanagerOPML.Enable(self.ipodder.config.feedmanager_enable)
        self.feedmanagerLbl.Enable(self.ipodder.config.feedmanager_enable)
        self.enableProxy.SetValue(self.ipodder.config.use_proxy_server)
        self.proxyPort.SetValue(self.ipodder.config.http_proxy_port)
        self.proxyHost.SetValue(self.ipodder.config.http_proxy_server)
        self.proxyUsername.SetValue(self.ipodder.config.http_proxy_username)	
        self.proxyPassword.SetValue(self.ipodder.config.http_proxy_password)
        if self.ipodder.config.use_proxy_server is False:
            self.proxyPort.Enable(False)
            self.proxyHost.Enable(False)
            self.proxyUsername.Enable(False)	
            self.proxyPassword.Enable(False)
        self.numFeedScanThreads.SetSelection(self.BestComboIntegerIndex(self.numFeedScanThreads,self.ipodder.config.max_scan_jobs))
        self.numDownloadThreads.SetSelection(self.BestComboIntegerIndex(self.numDownloadThreads,self.ipodder.config.max_download_jobs)) 
        self.maxHarddiscSizeMB.SetValue(str(self.ipodder.config.min_mb_free))
        if platform.system()=='Darwin':
            self.hideOnStartup.Enable(False)
        self.prefsSave.Enable(False)

    def BestComboIntegerIndex(self,combo,desired):
        """Return the index of the closest-valued integer.
        Useful when defaulting the selectlist to a possibly non-matching
        desired value."""
        idx = 0
        diff = None
        for i in range(0,combo.GetCount()):
            if diff == None or \
               abs(int(combo.GetString(i)) - desired) < diff:
                idx = i
                diff = abs(int(combo.GetString(i)) - desired)

        return idx

    def OnFeedmanagerEnableChk(self, event):
        self.feedmanagerOPML.Enable(event.IsChecked())
        self.feedmanagerLbl.Enable(event.IsChecked())
        self.OnPrefChange(event)

    def OnShowLogPage(self, event):
        """Adjust the configuration and hide or show the log page."""
        if event.IsChecked(): 
            self.ipodder.config.show_log_page = True
        else: 
            self.ipodder.config.show_log_page = False
        self.ipodder.config.flush()

    def OnPrefsSave(self, event):
        #validate the download directory
        if not os.path.isdir(self.downloaddir.GetValue()):
            message1 = self._("str_bad_directory_pref_1")
            message2 = self._("str_bad_directory_pref_2")
            alert = wx.MessageDialog(self, "%s (%s).  %s" % (message1,self.downloaddir.GetValue(),message2) , style=wx.OK)
            response = alert.ShowModal()
            alert.Destroy()
            return

        #validate the integer-valued configuration options
        try:
            int(self.maxHarddiscSizeMB.GetValue())    
        except ValueError:
            message1 = self._("str_bad_megabyte_limit_1")
            message2 = self._("str_bad_megabyte_limit_2")
            alert = wx.MessageDialog(self, "%s (%s).  %s" % (self._("str_bad_megabyte_limit_1"), self.maxHarddiscSizeMB.GetValue(), self._("str_bad_megabyte_limit_2")), style=wx.OK)
            response = alert.ShowModal()
            alert.Destroy()            
            return

        #validate the proxy settings
        if self.enableProxy.IsChecked() and (
            self.proxyHost.GetValue().strip() == '' or
            self.proxyHost.GetValue().strip() == 'http://' or
            self.proxyPort.GetValue().strip() == ''):
            alert = wx.MessageDialog(self, self._("str_bad_proxy_pref"), style=wx.OK)
            response = alert.ShowModal()
            alert.Destroy()
            return

        if self.enableProxy.IsChecked() and (
            self.proxyUsername.GetValue().strip() != '' and
            self.proxyPassword.GetValue().strip() == '' ):
            alert = wx.MessageDialog(self, self._("str_missing_proxy_password"), style=wx.OK)
            response = alert.ShowModal()
            alert.Destroy()
            return
 

        #validate the feed manager url
        if self.feedmanager_enable.IsChecked() and not \
           self.feedmanagerOPML.GetValue().strip().startswith('http://'):
            alert = wx.MessageDialog(self, self._("str_bad_feedmanager_url"), style=wx.OK)
            response = alert.ShowModal()
            alert.Destroy()
            self.feedmanagerOPML.SetValue(self.ipodder.config.feedmanager_opml_url)
            self.feedmanagerOPML.SetFocus()
            self.feedmanagerOPML.SelectAll()
            return

        new_manager = False
        if self.feedmanager_enable.IsChecked() and \
            ( (self.ipodder.config.feedmanager_opml_url != self.feedmanagerOPML.GetValue().strip()) or \
              not self.ipodder.config.feedmanager_enable
              ):
            new_manager = True

        if self.feedmanager_enable.IsChecked() and \
            ( (self.ipodder.config.feedmanager_opml_url != self.feedmanagerOPML.GetValue().strip()) or \
              not self.ipodder.config.feedmanager_enable
              ):
            new_manager = True

        if (self.ipodder.config.feedmanager_enable and \
            not self.feedmanager_enable.IsChecked()):
            self.gui.RemoveManagedFeeds()
        
        self.ipodder.config.download_dir = self.downloaddir.GetValue()
        
        self.ipodder.config.hide_on_startup = self.hideOnStartup.IsChecked()
        self.ipodder.config.hide_tray_icon = self.hideTrayIcon.IsChecked()
        self.ipodder.config.scan_on_startup = self.scanOnStartup.IsChecked()
        self.ipodder.config.chkupdate_on_startup = self.chkupdateOnStartup.IsChecked()
        self.ipodder.config.play_on_download = self.playOnDownload.IsChecked()
        self.ipodder.config.dl_command_enable = self.dlCommandEnable.IsChecked()
        self.ipodder.config.dl_command = self.dlCommand.GetValue()
        self.ipodder.config.player_command = self.playerCommand.GetValue()
        self.ipodder.config.show_log_page = self.showLogPage.IsChecked()            
        self.ipodder.config.max_scan_jobs = int(self.numFeedScanThreads.GetValue())
        self.ipodder.config.max_download_jobs = int(self.numDownloadThreads.GetValue())
        self.ipodder.config.min_mb_free = int(self.maxHarddiscSizeMB.GetValue())
        self.ipodder.config.coralize_urls = self.corlizeUrls.IsChecked()

        self.ipodder.config.use_proxy_server = self.enableProxy.IsChecked()
        self.ipodder.config.http_proxy_server = self.proxyHost.GetValue().strip()
        self.ipodder.config.http_proxy_port =  self.proxyPort.GetValue().strip()
        self.ipodder.config.http_proxy_username = self.proxyUsername.GetValue().strip()
        self.ipodder.config.http_proxy_password = self.proxyPassword.GetValue().strip()

        self.ipodder.config.feedmanager_enable = self.feedmanager_enable.IsChecked()
        self.ipodder.config.feedmanager_opml_url = self.feedmanagerOPML.GetValue().strip()
        self.ipodder.config.flush()
        if self.ipodder.config.feedmanager_enable and new_manager:
            self.gui.ReplaceManagedFeeds()
        self.gui.showlogpage(self.ipodder.config.show_log_page)
        self.ipodder.init_proxy_config()
        self.prefsSave.Enable(False)
        self.Show(0)
        
    def OnPrefChange(self, event):
        if not self.prefsSave.IsEnabled():
            self.prefsSave.Enable(True)
    
    def OnPrefsCancel(self,event):
        self.Show(0)
        self.ResetPrefs()

    def OnPrefsBrowse(self,event):
        self.OnPrefChange(event)
        dlg = wx.DirDialog(self,"Choose download directory",self.downloaddir.GetValue())
        if dlg.ShowModal() == wx.ID_OK:
            self.downloaddir.SetValue(dlg.GetPath())
        dlg.Destroy()       
        
class MyURLDropTarget(wx.PyDropTarget):
    def __init__(self, window):
        wx.PyDropTarget.__init__(self)
        self.window = window
        self.data = wx.URLDataObject();
        self.SetDataObject(self.data)

    def OnDragOver(self, x, y, d):
        return wx.DragLink

    def OnData(self, x, y, d):
        if not self.GetData():
            return wx.DragNone
        url = self.data.GetURL()
        self.window.AppendText(url + "\n")
        return d
